﻿using AutoFixture;
using Castle.DynamicProxy;
using Hotline.Api.Controllers;
using Hotline.Application.CallCenter;
using Hotline.Application.Tests.Dto;
using Hotline.Application.Tests.Infrastructure;
using Hotline.Application.Tests.Mock;
using Hotline.Application.Tests.SqlSuger;
using Hotline.Caching.Interfaces;
using Hotline.Caching.Services;
using Hotline.CallCenter.Calls;
using Hotline.FlowEngine.Notifications;
using Hotline.FlowEngine.WorkflowModules;
using Hotline.Identity.Accounts;
using Hotline.Identity.Roles;
using Hotline.Orders;
using Hotline.Repository.SqlSugar;
using Hotline.Settings;
using Hotline.Settings.Hotspots;
using Hotline.Share.Dtos.File;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Dtos.Order.Publish;
using Hotline.Share.Dtos.TrCallCenter;
using Hotline.Share.Dtos.Users;
using Hotline.Share.Enums.CallCenter;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Settings;
using Hotline.Share.Tools;
using Hotline.Snapshot.Interfaces;
using Hotline.Users;
using MediatR;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Testing;
using Microsoft.Extensions.DependencyInjection;
using Moq;
using Shouldly;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using XF.Domain.Authentications;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Application.Tests.Controller;
public class OrderControllerTest : TestBase
{
    private readonly OrderController _orderController;
    private readonly IRepository<Hotspot> _hotspotRepository;
    private readonly IOrderRepository _orderRepository;
    private readonly OrderServiceMock _orderServiceMock;
    private readonly IRepository<OrderPublish> _orderPublishRepository;
    private readonly INotificationHandler<EndWorkflowNotify> _orderPublishEndWorkflowHandler;
    private readonly IOrderVisitRepository _orderVisitRepository;
    private readonly IRepository<SystemSetting> _systemSettingRepository;
    private readonly ISystemSettingCacheManager _systemSettingCacheManager;
    private readonly IRepository<CallNative> _callNativeRepository;
    private readonly IRepository<CallidRelation> _callIdRelationRepository;
    private readonly XingTangCallApplication _defaultCallApplication;
    private readonly ISqlSugarClient _capSqlClient;

    public OrderControllerTest(IAccountRepository accountRepository, IRepository<Role> roleRepository, UserController userController, IRepository<Hotspot> hotspotRepository, OrderController orderController, IOrderRepository orderRepository, IServiceScopeFactory scopeFactory, IRepository<User> userRepository, OrderServiceMock orderServiceMock, IRepository<OrderPublish> orderPublishRepository, INotificationHandler<EndWorkflowNotify> orderPublishEndWorkflowHandler, IOrderVisitRepository orderVisitRepository, IRepository<SystemSetting> systemSettingRepository, ISystemSettingCacheManager systemSettingCacheManager, IRepository<CallNative> callNativeRepository, IRepository<CallidRelation> callIdRelationRepository, XingTangCallApplication defaultCallApplication, ISugarUnitOfWork<CapDbContext> capDbContext, IHttpContextAccessor httpContextAccessor, IThirdIdentiyService thirdService, IThirdAccountRepository thirdAccount) : base(accountRepository, roleRepository, userController, scopeFactory, userRepository, httpContextAccessor, thirdService, thirdAccount)
    {
        _hotspotRepository = hotspotRepository;
        _orderController = orderController;
        _orderController.ControllerContext = new ControllerContext
        {
            HttpContext = new DefaultHttpContext()
        };
        _orderRepository = orderRepository;
        _orderServiceMock = orderServiceMock;
        _orderPublishRepository = orderPublishRepository;
        _orderPublishEndWorkflowHandler = orderPublishEndWorkflowHandler;
        _orderVisitRepository = orderVisitRepository;
        _systemSettingRepository = systemSettingRepository;
        _systemSettingCacheManager = systemSettingCacheManager;
        _callNativeRepository = callNativeRepository;
        _callIdRelationRepository = callIdRelationRepository;
        _defaultCallApplication = defaultCallApplication;
        _capSqlClient = capDbContext.Db;
    }

    /// <summary>
    /// 创建工单时传入一个错误callId
    /// 验证: 
    /// 测试在延迟消息后会不会把工单测CallId修复成正确的CallId
    /// 推送的省上数据是否正确
    /// 测试修复后的CallId 再次保存功能, 能通过 systemLog 查询修复记录, 拦截工单保存
    /// </summary>
    /// <returns></returns>
    [Fact]
    public async Task OrderCallIdRelevance_Test()
    {
        var callNo = DateTime.Now.ToString("yyyyMMddhhmmss") + "Flow";
        var inDto = _fixture.Build<CallNative>()
            .With(m => m.Id, Ulid.NewUlid().ToString())
            .With(m => m.CallNo, callNo)
            .With(m => m.Direction, ECallDirection.In)
            .With(m => m.IsDeleted , false)
            .Create();
        await _callNativeRepository.AddAsync(inDto);

        var inDto2 = _fixture.Build<CallNative>()
            .With(m => m.Id, Ulid.NewUlid().ToString())
            .With(m => m.Direction, ECallDirection.Out)
            .With(m => m.CallNo, callNo)
            .With(m => m.AudioFile, string.Empty)
            .With(m => m.IsDeleted , false)
            .Create();
        await _callNativeRepository.AddAsync(inDto2);
        var callOrder = new CallidRelation
        {
            Id = callNo,
            CallId = inDto2.Id,
        };
        await _callIdRelationRepository.AddAsync(callOrder, CancellationToken.None);

        SetZuoXi();
        var order = _orderServiceMock.CreateOrder(callNo)
            .办理到派单员()
            .GetCreateResult();
        order.ShouldNotBeNull();
        await _defaultCallApplication.OrderRelateCallHandlerAsync(order.Id, CancellationToken.None);
        var orderEntity = await _orderRepository.GetAsync(order.Id);
        orderEntity.ShouldNotBeNull();
        orderEntity.CallId.ShouldBe(inDto.Id);
        var query = _capSqlClient.Queryable<PublishedEntity>()
            .Where(m => m.Name == "hotline.call.connect.order" && m.Content.Contains(order.Id));
        var sql = query.ToSqlString();
        sql.Contains("id").ShouldBeFalse("字段大小写错误" + sql);
        var published = await query.FirstAsync();
        published.ShouldNotBeNull("推送数据不存在");
        var message = published.Content.FromJson<Message<PublishCallRecrodDto>>();
        message.Value.Order.ShouldNotBeNull("推送的工单信息为空");
        message.Value.Order.ProvinceNo.ShouldNotBeNull("省工单编号不能为空");
        message.Value.Order.No.ShouldBe(order.No, "推送的工单编号错误");

        try
        {
            order = _orderServiceMock.CreateOrder(callNo)
                       .办理到派单员()
                       .GetCreateResult();
        }
        catch (UserFriendlyException e)
        {
            e.FriendlyMessage.ShouldBe("来电已保存工单!");
        }
    }

    /// <summary>
    /// 创建工单并派送给派单员
    /// </summary>
    /// <returns></returns>
    [Fact]
    public async Task ToPaiDanYuan_Test()
    {
        SetZuoXi();
        var order = _orderServiceMock.CreateOrder()
            .办理到派单员()
            .GetCreateResult();
        order.ShouldNotBeNull();
    }

    /// <summary>
    /// 验证中心直办工单归档后 自动发布
    /// 是否推诿, 是否不积极
    /// </summary>
    /// <returns></returns>
    [Fact]
    public async Task AutoPublish_Test()
    {
        SetZuoXi();
        var order = _orderServiceMock.CreateOrder()
            .办理到派单员()
            .办理到归档(SetPaiDanYuan, data =>
            {
                data.IsEvasive = true;
                data.IsInactively = true;
            })
            .GetCreateResult();
        var publish = await _orderPublishRepository.GetAsync(m => m.No == order.No);
        publish.ShouldNotBeNull(order.No);
        var orderEntity = await _orderRepository.GetAsync(order.Id);
        orderEntity.ShouldNotBeNull();
        orderEntity.IsEvasive.ShouldBeTrue();
        orderEntity.IsInactively.ShouldBeTrue();
        orderEntity.Status.ShouldBe(EOrderStatus.Visited);
        orderEntity.OrgProcessingResults.ShouldNotBeNull();
        orderEntity.OrgProcessingResults.Key.ShouldBe("4");
        orderEntity.OrgProcessingResults.Value.ShouldBe("满意");
        orderEntity.SeatEvaluate.ShouldBe(ESeatEvaluate.Satisfied);
    }

    [Fact]
    public void Set一级部门_Test()
    {
        Set一级部门();
    }

    [Fact]
    public async Task IsStepUrgent_True_Test()
    {
        SetZuoXi();
        var order = _orderServiceMock.CreateOrder()
            .办理到派单员()
            .办理到归档(SetPaiDanYuan, data => data.IsStepUrgent = true, workflow =>
            {
                workflow.BusinessType = EBusinessType.DepartmentLeader;
                workflow.Opinion = "测试保存勾选紧急";
            })
            .GetCreateResult();
        order.ShouldNotBeNull();
        order.Id.ShouldNotBeNull();
        var orderDto = await _orderController.Get(order.Id);
        orderDto.IsStepUrgent.ShouldBeTrue();
        orderDto.IsStepUrgenText.ShouldBe("紧急");
    }

    /// <summary>
    /// 测试在办理的过程中能正确保存 
    /// 1.与市民电话联系
    /// 2.已赴现场处置
    /// 3.其他
    /// 4.其他原因
    /// 5.经办人
    /// 6.经办人电话
    /// 7.沟通时间
    /// 8.沟通地点
    /// </summary>
    /// <returns></returns>
    [Fact]
    public async Task CreateOrder_Test()
    {
        SetZuoXi();
        var order = _orderServiceMock.CreateOrder()
            .办理到一级部门()
            .办理到二级部门(Set一级部门)
            .办理一级部门汇总(Set二级部门)
            .GetCreateResult();
        order.Id.ShouldNotBeNull();
        var orderEntity = await _orderRepository.GetAsync(order.Id);
        orderEntity.RealCommunicationAddress.ShouldNotBeNull();
        orderEntity.RealCommunicationTime.ShouldNotBeNull();
        orderEntity.RealContactLocale.ShouldNotBeNull();
        orderEntity.RealContactLocale.ShouldBe(true);
        orderEntity.RealHandlerName.ShouldNotBeNull();
        orderEntity.RealHandlerPhone.ShouldNotBeNull();
    }

    [Fact]
    public async Task GetOrderHandleTimeConfigByAcceptType_Test()
    {
        var result = await _orderController.GetOrderHandleTimeConfigByAcceptType(Share.Enums.FlowEngine.EFlowDirection.OrgToCenter, "10");
        result.Count.ShouldBe(1);
        result.TimeText.ShouldBe("1个工作日");
        result.TimeType.ShouldBe(ETimeType.WorkDay);
    }


    /// <summary>
    /// 测试取消发布功能
    /// </summary>
    /// <returns></returns>
    [Fact]
    public async Task CancelOrderPublish_Test()
    {
        var system = await _systemSettingRepository.Queryable()
            .Where(m => m.Code == SettingConstants.CancelPublishOrderEnabled)
            .FirstAsync();
        system.SettingValue[0] = "true";
        await _systemSettingRepository.UpdateAsync(system);
        _systemSettingCacheManager.DelSystemSetting(SettingConstants.CancelPublishOrderEnabled);

        SetZuoXi();
        var order = _orderServiceMock.CreateOrder()
            .办理到派单员()
            .办理到归档(SetPaiDanYuan, data =>
            {
                data.IsEvasive = true;
                data.IsInactively = true;
            })
            .GetCreateResult();
        var publish = await _orderPublishRepository.GetAsync(m => m.No == order.No);
        publish.ShouldNotBeNull(order.No);
        var orderEntity = await _orderRepository.GetAsync(order.Id);
        orderEntity.ShouldNotBeNull();
        orderEntity.IsEvasive.ShouldBeTrue();
        orderEntity.IsInactively.ShouldBeTrue();

        await _orderController.PublishCancelAsync(new PublishCancelInDto { OrderPublishId = publish.Id });
        var visit = await _orderVisitRepository.GetAsync(m => m.OrderId == order.Id);
        var publishCount = await _orderPublishRepository.Queryable().Where(m => m.OrderId == order.Id && m.IsDeleted == false).CountAsync();
        publishCount.ShouldBe(0, "发布工单数据已经被删除, 应该是0条");
        visit.ShouldNotBeNull("回访信息不存在");
        visit.VisitState.ShouldBe(EVisitState.Visited, "回访状态应该是已回访");

        orderEntity = await _orderRepository.GetAsync(order.Id);
        orderEntity.Status.ShouldBe(EOrderStatus.Filed);
        var publishList = await _orderController.PublishOrderList(new QueryOrderPublishDto
        {
            PageSize = 1000,
            QuerySelf = true
        });

        publishList.Items.Any(m => m.Id == order.Id).ShouldBeTrue("工单发布列表中没有取消的工单");
        SetZuoXi();
        // 验证是否能非上次发布人员发布
        try
        {
            await _orderController.PublishOrder(new PublishOrderDto { Id = order.Id });
        }
        catch (UserFriendlyException e)
        {
            e.Message.Contains("您不能发布").ShouldBeTrue();
        }

        // 验证被取消前的发布人能否继续发布
        SetPaiDanYuan();
        var inDto = _fixture.Create<PublishOrderDto>();
        inDto.Id = order.Id;
        await _orderController.PublishOrder(inDto);
        var published = await _orderPublishRepository.Queryable().Where(m => m.OrderId == order.Id && m.IsDeleted == false).FirstAsync();
        published.ShouldNotBeNull();
        published.IsDeleted.ShouldBeFalse();
    }
}
