﻿using Hotline.Share.Enums;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Snapshot;
using Hotline.Share.Requests;
using System.ComponentModel.DataAnnotations;
using XF.Utility.EnumExtensions;

namespace Hotline.Share.Dtos.Snapshot;

public class OrderInDto : QueryFixedDto
{
    /// <summary>
    /// 关键字
    /// </summary>
    public string? KeyWords { get; set; }

    /// <summary>
    /// 工单状态;
    /// 0: 全部;
    /// 1: 未回复;
    /// 2: 已回复;
    /// 3: 已评价;
    /// </summary>
    public EOrderQueryStatus Status { get; set; }
}

public class OrderDetailOutDto : OrderOutDto
{
}

public class OrderOutDto
{
    public string Id { get; set; }

    /// <summary>
    /// 工单编号
    /// </summary>
    public string OrderNo { get; set; }

    /// <summary>
    /// 标题
    /// </summary>
    public string Title { get; set; }

    /// <summary>
    /// 工单状态
    /// </summary>
    public EOrderStatus Status { get; set; }

    /// <summary>
    /// 行业名字
    /// </summary>
    public string IndustryName { get; set; }

    /// <summary>
    /// 工单状态描述
    /// </summary>
    public string StatusText
    {
        get
        {
            if (Status == EOrderStatus.Published || Status == EOrderStatus.Filed)
            {
                return "办理完成";
            }
            return Status.GetDescription();
        }
    }

    /// <summary>
    /// 时间
    /// </summary>
    public DateTime CreationTime { get; set; }

    /// <summary>
    /// 时间文本
    /// </summary>
    public string CreationTimeText => this.CreationTime.ToString("yyyy-MM-dd hh:mm:ss");

    /// <summary>
    /// 区域
    /// </summary>
    public string Area { get; set; }
}
public class OrderDto
{
}

public class AddSnapshotOrderOutDto
{
    public string Id { get; set; }
    public string No { get; set; }
    public string Password { get; set; }
}

public class AddSnapshotOrderInDto : Position
{
    /// <summary>
    /// 行业Id
    /// </summary>
    [Required(ErrorMessage = "行业Id不能为空")]
    public string IndustryId { get; set; }

    /// <summary>
    /// 作业类型
    /// </summary>
    public string? JobType { get; set; }

    /// <summary>
    /// 经营单位类别
    /// </summary>
    public string? BusinessUnitType { get; set; }

    /// <summary>
    /// 作业场所
    /// </summary>
    public string? Workplace { get; set; }

    /// <summary>
    /// 场所名称(多个场所使用 - 连接)
    /// </summary>
    public string? WorkplaceName { get; set; }

    /// <summary>
    /// 作业时间
    /// </summary>
    public DateTime? StartWorkTime { get; set; }

    /// <summary>
    /// 作业结束时间
    /// </summary>
    public DateTime? EndWorkTime { get; set; }

    /// <summary>
    /// 姓名
    /// </summary>
    [Required(ErrorMessage = "姓名不能为空")]
    public string Name { get; set; }

    /// <summary>
    /// 联系方式
    /// </summary>
    [Required(ErrorMessage = "联系方式不能为空")]
    public string PhoneNumber { get; set; }

    /// <summary>
    /// 是否保密
    /// </summary>
    public bool? IsSecret { get; set; }

    /// <summary>
    /// 附件信息
    /// </summary>
    public IList<SnapshotFileInDto> Files { get; set; }

    /// <summary>
    /// 事件描述
    /// </summary>
    public string? Description { get; set; }

    public string GetContent(EIndustryType industryType)
    {
        if (industryType == EIndustryType.Declare)
        {
            return $"经营单位类别: {this.BusinessUnitType}\r\n" + // 个人
                    $"作业场所:{this.Workplace}\r\n" + // 化工、民爆物品生产经营企业
                    $"场所名称:{this.WorkplaceName}\r\n" + // 多业态混合经营场所 - 存储多种功能的劳动密集型企业
                    $"作业区域: {this.County + this.Town}\r\n" + // 自流井区舒坪街道
                    $"作业类型: {this.JobType}\r\n" + // 电焊
                    $"作业时间:{this.StartWorkTime.Value.ToString("yyyy-MM-dd hh:mm")}~{this.EndWorkTime.Value.ToString("yyyy-MM-dd hh:mm")}\r\n" +
                    $"作业地点: {this.Town}({this.County})（{this.Street}）"; // 舒平(自流井区)（油库）"
        }
        return $"{Description}";
    }

    public string GetTitle(EIndustryType industryType, string acceptType)
    {
        if (industryType == EIndustryType.Declare)
        {
            return $"【随手拍】关于{Town}({County})的申报";
        }

        return $"【随手拍】关于{Town}({County})的" + acceptType;
    }
}


/// <summary>
/// 红包审核入参
/// </summary>
/// <param name="No"></param>
/// <param name="Title"></param>
/// <param name="FromPhone">来电人电话</param>
/// <param name="BeginCreationTime">受理开始时间</param>
/// <param name="EndCreationTime">受理结束时间</param>
/// <param name="IsDeal">是否办理</param>
/// <param name="IsTruth">是否属实</param>
/// <param name="BeginAuditTime">审批开始时间</param>
/// <param name="EndAuditTime">审批结束时间</param>
/// <param name="IsIssued">是否发放红包</param>
/// <param name="IndustryId">行业Id</param>
/// <param name="ConfigAmount">配置金额</param>
/// <param name="AcutalAmount">实际发放金额</param>
/// <param name="ApprovedAmount">审批金额</param>
/// <param name="IsDanger">是否安全生成</param>
public record SnapshotOrderAuditItemsInDto(string? No, string? Title,
    string? FromPhone,
    DateTime? BeginCreationTime,
    DateTime? EndCreationTime,
    bool? IsDeal,
    bool? IsTruth,
    DateTime? BeginAuditTime,
    DateTime? EndAuditTime,
    bool? IsIssued,
    string? IndustryId,
    double? ConfigAmount,
    double? AcutalAmount,
    double? ApprovedAmount,
    bool? IsDanger) : PagedRequest;

public class SnapshotOrderAuditItemsOutDto
{
    /// <summary>
    /// No
    /// </summary>
    public string No { get; set; }

    /// <summary>
    /// 标题
    /// </summary>
    public string Title { get; set; }

    /// <summary>
    /// 行业
    /// </summary>
    public string IndustryName { get; set; }

    /// <summary>
    /// 行业Id
    /// </summary>
    public string IndustryId { get; set; }

    /// <summary>
    /// 来源渠道（电话、网站、APP等）
    /// </summary>
    public string? SourceChannel { get; set; }

    /// <summary>
    /// 来源渠道代码
    /// </summary>
    public string? SourceChannelCode { get; set; }

    /// <summary>
    /// 工单状态
    /// </summary>
    public EOrderStatus Status { get; set; }

    /// <summary>
    /// 状态
    /// </summary>
    public string StatusTxt => Status.GetDescription();

    /// <summary>
    /// 线索分类
    /// </summary>
    public string IndustryCase { get; set; }

    /// <summary>
    /// 线索分类
    /// </summary>
    public string IndustryCaseId { get; set; }

    /// <summary>
    /// 标记为安全生产
    /// </summary>
    public bool? IsDanger { get; set; }

    /// <summary>
    /// 来电号码
    /// </summary>
    public string? FromPhone { get; set; }

    /// <summary>
    /// 来电/信人姓名
    /// </summary>
    public string? FromName { get; set; }

    /// <summary>
    /// 审批时间
    /// </summary>
    public DateTime? AuditTime { get; set; }

    /// <summary>
    /// 审批金额
    /// </summary>
    public double? ApprovedAmount { get; set; }

    /// <summary>
    /// 实发金额
    /// </summary>
    public double? AcutalAmount { get; set; }

    /// <summary>
    /// 补充发放金额
    /// TODO: 没有逻辑
    /// </summary>
    public double? ReplenishAmount { get; set; }

    /// <summary>
    /// 市民奖励发放结果
    /// 是否已发放
    /// </summary>
    public bool? IsIssued { get; set; }

    /// <summary>
    /// 市民奖励
    /// </summary>
    public string CitizenAward { get; set; }

    /// <summary>
    /// 市民奖励发放备注
    /// </summary>
    public string RecordRemark { get; set; }

    /// <summary>
    /// 区域
    /// </summary>
    public string County { get; set; }

    /// <summary>
    /// 部门是否整改完成;
    /// 是否整改
    /// </summary>
    public bool IsRectify { get; set; }

    /// <summary>
    /// 网格员是否办理
    /// </summary>
    public bool? IsDeal { get; set; }

    /// <summary>
    /// 网格E通编号
    /// </summary>
    public string? NetworkENumber { get; set; }

    /// <summary>
    /// 是否属实
    /// </summary>
    public bool? IsTruth { get; set; }

    /// <summary>
    /// 是否重复
    /// </summary>
    public bool? IsRepetition { get; set; }

    /// <summary>
    /// 受理时间
    /// </summary>
    public DateTime CreationTime { get; set; }

    /// <summary>
    /// 一级部门code
    /// </summary>
    public string? OrgLevelOneCode { get; set; }

    /// <summary>
    /// 一级部门名称
    /// </summary>
    public string? OrgLevelOneName { get; set; }

    /// <summary>
    /// 审批人
    /// </summary>
    public string? AuditId { get; set; }

    /// <summary>
    /// 审批人
    /// </summary>
    public string? AuditName { get; set; }

    /// <summary>
    /// 审批部门
    /// </summary>
    public string? AuditOrgId { get; set; }

    /// <summary>
    /// 审批部门名称
    /// </summary>
    public string? AuditOrgName { get; set; }

    /// <summary>
    /// 审批意见
    /// </summary>
    public string? AuditRemark { get; set; }

    /// <summary>
    /// 银行卡号
    /// </summary>
    public string? BankCardNo { get; set; }

    /// <summary>
    /// 开户行
    /// </summary>
    public string? OpenBank { get; set; }
}
