﻿using DotNetCore.CAP;
using Hotline.Caching.Interfaces;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Dtos.Push;
using Hotline.Share.Enums.Order;
using Hotline.Share.Tools;
using Mapster;
using Microsoft.Extensions.Logging;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;
using XF.Domain.Repository;
using Hotline.EventBus;
using Hotline.Orders.Notifications;

namespace Hotline.Orders;
public class OrderVisitDomainService : IOrderVisitDomainService, IScopeDependency
{
    private readonly IRepository<OrderVisitDetail> _orderVisitDetailRepository;
    private readonly ILogger<OrderVisitDomainService> _logger;
    private readonly IRepository<Order> _orderRepository;
    private readonly ISystemDicDataCacheManager _systemDicDataCacheManager;
    private readonly IOrderVisitRepository _orderVisitRepository;
    private readonly ICapPublisher _capPublisher;
    private readonly Publisher _publisher;

    public OrderVisitDomainService(IRepository<OrderVisitDetail> orderVisitDetailRepository, ILogger<OrderVisitDomainService> logger, IRepository<Order> orderRepository, ISystemDicDataCacheManager systemDicDataCacheManager, IOrderVisitRepository orderVisitRepository, ICapPublisher capPublisher, Publisher publisher)
    {
        _orderVisitDetailRepository = orderVisitDetailRepository;
        _logger = logger;
        _orderRepository = orderRepository;
        _systemDicDataCacheManager = systemDicDataCacheManager;
        _orderVisitRepository = orderVisitRepository;
        _capPublisher = capPublisher;
        _publisher = publisher;
    }

    /// <summary>
    /// 用户回访短信回复更新回访状态
    /// </summary>
    /// <param name="dto"></param>
    /// <param name="data"></param>
    /// <returns></returns>
    public async Task UpdateSmsReplyAsync(MessageDto data)
    {
        if (data.IsSmsReply == false || data.SmsReplyContent.IsNullOrEmpty()) return;

        var orderVisits = await _orderVisitRepository.Queryable()
            .Includes(m => m.Order)
            .Where(m => m.Order.Contact == data.TelNumber)
            .Where(m => m.VisitState == EVisitState.SMSVisiting)
            .ToListAsync();
            
        foreach (var orderVisit in orderVisits)
        {
            await UpdateSmsReplyAsync(orderVisit, data.SmsReplyContent!.Trim());
        }
    }

    /// <summary>
    /// 根据用户短信或电话回复的内容转换成系统中对应的评价
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <param name="replyTxt"></param>
    /// <returns></returns>
    /// <exception cref="UserFriendlyException"></exception>
    public T GetVisitEvaluateByReplyTxt<T>(string replyTxt)
    {
        Dictionary<string, string> ReplyToEnumMap = new()
        {
            { "1", $"非常满意|{EVisitState.Visited}|{ESeatEvaluate.VerySatisfied}|{EVoiceEvaluate.VerySatisfied}|5" },
            { "2", $"满意|{EVisitState.Visited}|{ESeatEvaluate.Satisfied}|{EVoiceEvaluate.Satisfied}|4"},
            { "3", $"一般|{EVisitState.Visited}|{ESeatEvaluate.Normal}|{EVoiceEvaluate.Normal}|4"},
            { "4", $"不满意|{EVisitState.SMSUnsatisfied}|{ESeatEvaluate.NoSatisfied}|{EVoiceEvaluate.NoSatisfied}|2"},
            { "5", $"非常不满意|{EVisitState.SMSUnsatisfied}|{ESeatEvaluate.NoSatisfied}|{EVoiceEvaluate.VeryNoSatisfied}|2"},
            { "0", $"默认满意|{EVisitState.Visited}|{ESeatEvaluate.DefaultSatisfied}|{EVoiceEvaluate.DefaultSatisfied}|0"},
        };
        replyTxt = replyTxt.Trim();
        if (ReplyToEnumMap.TryGetValue(replyTxt, out var result) == false)
        {
            var m = ReplyToEnumMap.FirstOrDefault(item => item.Value.StartsWith(replyTxt));
            replyTxt = m.Key;
            result = m.Value;
        }

        if (result.IsNullOrEmpty()) throw new UserFriendlyException($"用户回复内容异常; replyTxt: {replyTxt}");
        var replySplit = result.Split("|");

        if (typeof(T) == typeof(string))
            return (T)(replySplit[0] as object);

        if (typeof(T) == typeof(EVisitState))
            return (T)Enum.Parse(typeof(T), replySplit[1]);

        if (typeof(T) == typeof(ESeatEvaluate))
            return (T)Enum.Parse(typeof(T), replySplit[2]);

        if (typeof(T) == typeof(EVoiceEvaluate))
            return (T)Enum.Parse(typeof(T), replySplit[3]);

        if (typeof(T) == typeof(Kv))
            return _systemDicDataCacheManager.GetVisitSatisfaction()
           .First(m => m.DicDataValue == replySplit[4]).Adapt<T>();

        return default;
    }

    /// <summary>
    /// 超过48小时未回复短信,修改短信回访记录为默认满意
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    public async Task UpdateSmsReplyDefaultAsync(MessageDto dto)
    {
        await _orderVisitRepository.Queryable()
            .Includes(m => m.Order)
            .Where(m => m.Id == dto.ExternalId)
            .Where(m => m.VisitState == EVisitState.SMSVisiting)
            .FirstAsync()
            .Then(async orderVisit =>
            {
                // 默认满意
                await UpdateSmsReplyAsync(orderVisit, "0");
            });
    }

    /// <summary>
    /// 超过48小时未回复短信,修改短信回访记录为默认满意
    /// 用户回访短信回复更新回访状态
    /// </summary>
    /// <param name="orderVisit"></param>
    /// <param name="replyTxt"></param>
    /// <returns></returns>
    private async Task UpdateSmsReplyAsync(OrderVisit orderVisit, string replyTxt)
    {
        orderVisit.Order ??= await _orderRepository.GetAsync(orderVisit.OrderId);
        orderVisit.VisitType = EVisitType.SmsVisit;
        if (new string[] { "4", "5" , "不满意", "非常不满意" }.Contains(replyTxt))
        {
            // “短信不满意待回访”状态下，由其他方式再次进行回访，回访方式需更新为最新的回访方式
            // 故在此置为空
            orderVisit.VisitType = null;
        }
        var visitSatisfactionKv = GetVisitEvaluateByReplyTxt<Kv>(replyTxt);
        orderVisit.NowEvaluate = visitSatisfactionKv;

        orderVisit.VisitTime = DateTime.Now;
        orderVisit.VisitState = GetVisitEvaluateByReplyTxt<EVisitState>(replyTxt);
        await _orderVisitRepository.UpdateAsync(orderVisit, ignoreNullColumns: false);

        if (orderVisit.VisitState == EVisitState.Visited)
        {
            await _orderRepository.GetAsync(orderVisit.OrderId)
                .Then(async order =>
                {
                    order!.Visited(visitSatisfactionKv.Key, visitSatisfactionKv.Value);
                    await _orderRepository.UpdateAsync(order);
                });

        }
        var detailOrg = await _orderVisitDetailRepository.Queryable()
            .Where(m => m.VisitId == orderVisit.Id && m.VisitTarget == EVisitTarget.Org)
            .ToListAsync();
        foreach (var item in detailOrg)
        {
            item.ReplyBackfill(visitSatisfactionKv);
        }
        await _orderVisitDetailRepository.UpdateRangeAsync(detailOrg);

        await _orderVisitDetailRepository.Queryable()
            .Where(m => m.VisitId == orderVisit.Id && m.VisitTarget == EVisitTarget.Seat)
            .FirstAsync()
            .Then(async detailSeat =>
            {
                detailSeat.ReplyBackfill(GetVisitEvaluateByReplyTxt<ESeatEvaluate>(replyTxt));
                await _orderVisitDetailRepository.UpdateAsync(detailSeat);
            });


        if (orderVisit.VisitState != EVisitState.Visited) return;
        orderVisit.Order.Visited(visitSatisfactionKv.Key, visitSatisfactionKv.Value);
        await _orderRepository.UpdateAsync(orderVisit.Order);
        var orderDto = orderVisit.Order.Adapt<OrderDto>();
        var first = detailOrg.First();
        if (visitSatisfactionKv.Key != null && visitSatisfactionKv.Value != null)
        {
            //推省上
            await _capPublisher.PublishAsync(Hotline.Share.Mq.EventNames.HotlineOrderVisited,
                new PublishVisitDto()
                {
                    Order = orderDto,
                    No = orderVisit.No,
                    VisitType = orderVisit.VisitType,
                    VisitName = orderVisit.CreatorName,
                    VisitTime = orderVisit.VisitTime,
                    VisitRemark = string.IsNullOrEmpty(first.VisitContent) ? first.OrgProcessingResults?.Value : first.VisitContent,
                    AreaCode = orderVisit.Order.AreaCode!,
                    SubjectResultSatifyCode = first.OrgProcessingResults.Key,
                    FirstSatisfactionCode = orderVisit.Order.FirstVisitResultCode!,
                    ClientGuid = ""
                }, cancellationToken: CancellationToken.None);

        }

    }
}
